# plpgsql_sec

plpgsql_sec is an implementation of the PL/pgSQL procedural language that
encrypts and decrypts function bodies on the fly. It helps hiding
implementation of PL/pgSQL functions. The encryption/decryption uses the
pgcrypto module from PostgreSQL.

# INSTALL

```
PG_VER=15 # Set the desired PostgreSQL major version.
git clone https://github.com/cybertec-postgresql/plpgsql_sec.git
cd plpgsql_sec
git switch PG_${PG_VER}
make # `pg_config` must be in your $PATH.
     # alternatively: make PG_CONFIG=/usr/lib/postgresql/$PG_VER/bin/pg_config
make install # Possibly with `sudo`.
```

# SETUP

1. Install the extension, either build it yourself or the binary package.
2. Create or copy the encryption password, `plpgsql_sec_passwd` file.

If you're doing the setup on a new instance, use the `plpgsql_pwdgen` utility
to create the password file:

```
<PG installation>/bin/plpgsql_pwdgen <your data directory>
```

On the other hand, if your instance was initialized by `pg_upgrade`, you need
to copy the password file from the old instance.

However, if you have upgraded your instance from PostgreSQL server version 13
or lower, you need to extract the password from the old installation of the
plpgsql_sec extension rather than creating a new one. The easiest way is to
pass the old password file to the `plpgsql_pwdgen` tool:

```
<PG installation>/bin/plpgsql_pwdgen -o <old plpgsql_sec source>/passwd.h <your data directory>
```

Where "old plpgsql_sec source" stands for the directory in which you have the
old version of the plpgsql_sec extension. (The earlier versions could only be
installed from source code.)

In any case, the file `plpgsql_sec_passwd` should appear in the data directory.
Please check if it has the correct permissions, i.e. that only the user that
runs the instance can read it.

3. Start the instance
4. Connect to your database as superuser and run `CREATE EXTENSION plpgsql_sec;`

# USAGE

From this point you can pass `plpgsql_sec` language to the `CREATE FUNCTION`
command:

```
CREATE OR REPLACE FUNCTION func_name(... ) ... LANGUAGE plpgsql_sec;
```

The function body will now be stored encrypted.

If such a function is dumped and reloaded then the encryption scheme is
recognized and encryption is skipped, the function is stored with the already
provided encryption.

**CAUTION:** The encryption key is located in the data directory. Therefore,
the function text can potentially be decrypted by the superuser, as well as by
any user who has the `EXECUTE` permission on the appropriate file access
functions [1] or is granted the appropriate predefined role [2].

# REFERENCES

1. https://www.postgresql.org/docs/current/functions-admin.html#FUNCTIONS-ADMIN-GENFILE
2. https://www.postgresql.org/docs/current/predefined-roles.html
